/*
Licensed Materials - Property of IBM
5725-C94, 5725-C95, 5725-C96, 5655-Y02
© Copyright IBM Corporation 2012. All Rights Reserved.
This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer 
(a) for its own instruction and study, (b) in order to develop applications designed to run with an IBM product, 
either for customer's own internal use or for redistribution by customer, as part of such an application, in customer's 
own products.
*/

package com.ibm.bpm.integration.filenet;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import com.ibm.websphere.security.NotImplementedException;
import com.ibm.wsspi.security.auth.callback.Constants;
import com.ibm.wsspi.security.auth.callback.WSMappingCallbackHandlerFactory;

public class BPMClientConfig {
    
    public static final String BPM_SERVER_URI_PROPERTY = "bpm.server.uri";
    public static final String BPM_SERVER_PASSWORD_PROPERTY = "bpm.server.password";
    public static final String BPM_SERVER_USERNAME_PROPERTY = "bpm.server.username";
    public static final String BPM_SERVER_AUTH_ALIAS_PROPERTY = "bpm.server.authalias";
    
    private static final String CLASS = BPMEventHandler.class.getName();
    private Logger logger = Logger.getLogger(CLASS);
    private URI targetServer = null;
    private String userName = "UNAUTHENTICATED";
    private String password = null;
    private String authAlias = null;
    
    public BPMClientConfig(Properties props) {
        
        try {
            this.targetServer = new URI(props.getProperty(BPM_SERVER_URI_PROPERTY, "http://localhost:9080"));
        } catch (URISyntaxException e) {
            throw new IllegalArgumentException(e);
        }
        
        this.userName = props.getProperty(BPM_SERVER_USERNAME_PROPERTY, "UNAUTHENTICATED");
        this.password = props.getProperty(BPM_SERVER_PASSWORD_PROPERTY, "");
        this.authAlias = props.getProperty(BPM_SERVER_AUTH_ALIAS_PROPERTY);
        
        if (this.authAlias != null) {
            try {
                lookupCredentials();
            } catch (LoginException e) {
                this.userName = props.getProperty(BPM_SERVER_USERNAME_PROPERTY, "UNAUTHENTICATED");
                this.password = props.getProperty(BPM_SERVER_PASSWORD_PROPERTY, "");
            }
        }
    }

    @SuppressWarnings("unchecked")
    private void lookupCredentials() throws LoginException {
        String methodName = "lookupCredentials";
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, methodName);
        try {
            Map map = new HashMap();
            map.put(Constants.MAPPING_ALIAS, authAlias);
            // create a callback handler with the specified property (to find the alias) and null for the managed connection factory (MCF) since we don't need it
            CallbackHandler handler = WSMappingCallbackHandlerFactory.getInstance().getCallbackHandler(map, null);

            Subject subject = new Subject();
            LoginContext lc = new LoginContext("DefaultPrincipalMapping", subject, handler);
            if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "trying to login via authentication alias: {0}", authAlias );
            lc.login();
            subject = lc.getSubject();
            Set<PasswordCredential> pwdCredentialSet = subject.getPrivateCredentials(PasswordCredential.class);
            PasswordCredential pwCred = pwdCredentialSet.iterator().next();
            
            if (pwCred != null) {
                if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "successfully retrieved the PasswordCredentials" );
                byte[] passphrase = new String(pwCred.getPassword()).getBytes("UTF-8");
                setPassword(new String(passphrase, "UTF-8"));
                String userName = pwCred.getUserName();
                if (userName != null) {
                    if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "using username {0}, retrieved from the authentication alias", userName);
                    setUserName(userName);
                }
            }

            lc.logout();
            if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, methodName);
        } catch (NotImplementedException e) {
            throw new AssertionError();
        } catch (UnsupportedEncodingException e) {
            throw new AssertionError();
        }
    }

    /**
     * @return the password
     */
    String getPassword() {
        return password;
    }

    /**
     * @param password the password to set
     */
    public void setPassword(String password) {
        this.password = password;
    }

    public URI getTargetServer() {
        return targetServer;
    }

    public void setTargetServer(URI targetServer) {
        this.targetServer = targetServer;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer("TargetServer: '").append(getTargetServer().toString());
        if (authAlias != null) {
            buf.append("', AuthenticationAlias: '").append(authAlias);
        }
        buf.append("' , username: '").append(getUserName()).append("' , password: '********' ");
        buf.append(" objecthash: ").append(super.toString());
        return buf.toString();
    }
    
    
    

}
