/*
Licensed Materials - Property of IBM
5725-C94, 5725-C95, 5725-C96, 5655-Y02
© Copyright IBM Corporation 2012. All Rights Reserved.
This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer 
(a) for its own instruction and study, (b) in order to develop applications designed to run with an IBM product, 
either for customer's own internal use or for redistribution by customer, as part of such an application, in customer's 
own products.
*/

package com.ibm.bpm.integration.filenet;

import java.util.Properties;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.filenet.api.admin.ClassDefinition;
import com.filenet.api.collection.ContentElementList;
import com.filenet.api.constants.PropertyNames;
import com.filenet.api.core.ContentTransfer;
import com.filenet.api.core.Document;
import com.filenet.api.core.Factory;
import com.filenet.api.core.Folder;
import com.filenet.api.core.IndependentObject;
import com.filenet.api.core.ObjectStore;
import com.filenet.api.engine.EventActionHandler;
import com.filenet.api.events.CancelCheckoutEvent;
import com.filenet.api.events.ChangeClassEvent;
import com.filenet.api.events.ChangeStateEvent;
import com.filenet.api.events.CheckinEvent;
import com.filenet.api.events.CheckoutEvent;
import com.filenet.api.events.ClassifyCompleteEvent;
import com.filenet.api.events.CreationEvent;
import com.filenet.api.events.DeletionEvent;
import com.filenet.api.events.DemoteVersionEvent;
import com.filenet.api.events.FileEvent;
import com.filenet.api.events.FreezeEvent;
import com.filenet.api.events.LockEvent;
import com.filenet.api.events.ObjectChangeEvent;
import com.filenet.api.events.PromoteVersionEvent;
import com.filenet.api.events.PublishCompleteEvent;
import com.filenet.api.events.PublishRequestEvent;
import com.filenet.api.events.Subscription;
import com.filenet.api.events.UnfileEvent;
import com.filenet.api.events.UnlockEvent;
import com.filenet.api.events.UpdateEvent;
import com.filenet.api.events.UpdateSecurityEvent;
import com.filenet.api.exception.EngineRuntimeException;
import com.filenet.api.property.FilterElement;
import com.filenet.api.property.PropertyFilter;
import com.filenet.api.util.Id;

public class BPMEventHandler implements EventActionHandler {
    
    private static final String CMIS_DOCUMENT_SYMBOLIC_NAME = "cmis:document";
    private static final String CMIS_FOLDER_SYMBOLIC_NAME = "cmis:folder";
    
    private static final String CLASS = BPMEventHandler.class.getName();

    private Logger logger = Logger.getLogger(CLASS);

    private Properties properties = null;
    private String userString = null;
    
    private ECMEventPoster eventPoster = null;

    @Override
    public void onEvent(ObjectChangeEvent event, Id subId) throws EngineRuntimeException {
        
        String methodName = "onEvent";
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, methodName, new Object[]{event, subId});
        
        try {
            
            ObjectStore os = event.getObjectStore();

            //we have to fetch the ObjectStore again as some properties are not in cache yet.
            
            PropertyFilter objectStoreIDFilter = new PropertyFilter();
            objectStoreIDFilter.addIncludeProperty(new FilterElement(null, null, null, PropertyNames.ID, null));
            objectStoreIDFilter.addIncludeProperty(new FilterElement(null, null, null, PropertyNames.OBJECT_STORE_ID, null));
            os.fetchProperties(objectStoreIDFilter);
            
            String eventSourceId = os.get_ObjectStoreId().toString();
            
            Subscription subscription = Factory.Subscription.fetchInstance(os, subId, new PropertyFilter());

            // get the user string to obtain the object id of the properties file to load as configuration 
            userString = subscription.get_UserString();
          
            if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "fetched UserString: {0} ", userString );
            
            if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "Calculating event handler configuration based on UserString {0}", new Object[] { (this.userString)} );
            
            if (userString != null) {
                
                if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "trying to load config from document with objectId: {0}", userString);
            
                PropertyFilter pf = new PropertyFilter();
                pf.addIncludeProperty(new FilterElement(null, null, null, PropertyNames.CONTENT_SIZE, null));
                pf.addIncludeProperty(new FilterElement(null, null, null, PropertyNames.CONTENT_ELEMENTS, null));
                pf.addIncludeProperty(new FilterElement(null, null, null, PropertyNames.CURRENT_VERSION, null));
                pf.addIncludeProperty(new FilterElement(null, null, null, PropertyNames.IS_CURRENT_VERSION, null));
                
                Document configDoc = Factory.Document.fetchInstance(os, userString, pf);

                if (!configDoc.get_IsCurrentVersion()) {
                    if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "there's a newer version of the properties in the ObjectStore, trying to get the current version");
                    //get the most current version of properties document
                    //VersionSeries versions = Factory.VersionSeries.fetchInstance(os, configDoc.get_Id(), pf);
                    configDoc = (Document)configDoc.get_CurrentVersion();
                    if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "trying to load config from the current document with objectId: {0}", configDoc.get_Id());
                }
                // Get content elements and iterate list.
                ContentElementList docContentList = configDoc.get_ContentElements();
                properties = new Properties();
                ContentTransfer ct = (ContentTransfer) docContentList.get(0);
                properties.load(ct.accessContentStream());
                
            }
            else {
                if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "UserString is empty - unable to load configuration");
                throw new IllegalArgumentException("UserString == null");
            }

            if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, traceProperties());
            
            PropertyFilter pf = new PropertyFilter();
            pf.addIncludeProperty(new FilterElement(null, null, false, PropertyNames.ID, null));
            pf.addIncludeProperty(new FilterElement(null, null, false, PropertyNames.NAME, null));
            pf.addIncludeProperty(new FilterElement(null, null, false, PropertyNames.CLASS_IDENTITY, null));
            pf.addIncludeProperty(new FilterElement(null, null, false, PropertyNames.SYMBOLIC_NAME, null));
            
            ClassDefinition classDef = Factory.ClassDefinition.fetchInstance(os, event.get_SourceClassId(), pf);
            
            if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "id: {0}, symbolicName: {1}", new Object[]{classDef.get_Id(), classDef.get_SymbolicName()});
            
            String objectTypeId = classDef.get_SymbolicName();
            
            //we have to translate the well known doc/folder symbolic names to the format the cmis interface is using
            if ("Folder".equalsIgnoreCase(objectTypeId)) {
                objectTypeId = CMIS_FOLDER_SYMBOLIC_NAME;
                if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "detected predefined objetcID {0} with symbolicName {1}, mapping to 'cmis:folder'", new Object[]{classDef.get_Id(), classDef.get_SymbolicName()});
            }
            else if ("Document".equalsIgnoreCase(objectTypeId)) {
                objectTypeId = CMIS_DOCUMENT_SYMBOLIC_NAME;
                if (logger.isLoggable(Level.FINEST)) logger.log(Level.FINEST, "detected predefined objetcID {0} with symbolicName {1}, mapping to 'cmis:document'", new Object[]{classDef.get_Id(), classDef.get_SymbolicName()});
            }
                        
            ECMEvent ecmEvent = new ECMEvent(eventSourceId, getBPMEventType(event), objectTypeId, getCMISObjectID(event));
            int rc = getEventPoster().postEvent(ecmEvent);

            if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, methodName, rc);

         } catch (Exception e) {
             
            if (logger.isLoggable(Level.FINEST)) {
                logger.log(Level.FINE, "Exception when trying to post the event to BPM the ExceptionMessage was: {0}", e.getMessage());
                logger.log(Level.FINEST,"StackTrace: ", e);
            }
            throw new RuntimeException(e);
        }
    }
    
    private String getCMISObjectID(ObjectChangeEvent event) {
        String methodName = "getCMISObjectID";
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, methodName, event);
        
        IndependentObject sourceObject = event.get_SourceObject();
        String objectId = event.get_SourceObjectId().toString();
        StringBuffer cmisFormOID = new StringBuffer(40);
        
        //prefix with 'idf_' for objects with base type folder, or with 'idd_' for objects with base type document 
        if (sourceObject instanceof Folder) {
            cmisFormOID.append("idf_");
        }
        else if (sourceObject instanceof Document) {
            cmisFormOID.append("idd_");
        }
        
        cmisFormOID.append(objectId.substring(1, objectId.length()-1)); //strip curly brackets {}
     
        if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, methodName, cmisFormOID.toString());
        return cmisFormOID.toString();
    }

    
    private String traceProperties() {
      StringBuffer buf = new StringBuffer("[");
      for (Entry<Object, Object> entry  : properties.entrySet()) {
          buf.append(entry.getKey()).append(" = ");
          if (BPMClientConfig.BPM_SERVER_PASSWORD_PROPERTY.equals(entry.getKey())) {
              buf.append("********** ");
          }
          else {
              buf.append(entry.getValue()).append(" ");
          } 
      }
      buf.append("]\n");
      return buf.toString();
    }
 
    private ECMEventPoster getEventPoster() {
        String methodName = "getEventPoster";
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, methodName);
    
        BPMClientConfig config = new BPMClientConfig(properties);
        eventPoster = new ECMEventPoster(config);
    
        if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, methodName, eventPoster);
        return eventPoster;
        
    }

    private BPMEventType getBPMEventType(ObjectChangeEvent event) {
        String methodName = "getBPMEventType";
        if (logger.isLoggable(Level.FINEST)) logger.entering(CLASS, methodName, event);
        
        BPMEventType eventType = null;
        if (event instanceof CancelCheckoutEvent) {
            eventType = BPMEventType.CheckOutCanceled;
        } else if (event instanceof ChangeClassEvent) {
            eventType = BPMEventType.ClassChanged;
        } else if (event instanceof ChangeStateEvent) {
            eventType = BPMEventType.StateChanged;
        } else if (event instanceof CheckinEvent) {
            eventType = BPMEventType.CheckedIn;
        } else if (event instanceof CheckoutEvent) {
            eventType = BPMEventType.CheckedOut;
        } else if (event instanceof ClassifyCompleteEvent) {
            eventType = BPMEventType.ClassifyCompleted;        
        } else if (event instanceof CreationEvent) {
            eventType = BPMEventType.Created;
        } else if (event instanceof DeletionEvent) {
            eventType = BPMEventType.Deleted;
        } else if (event instanceof DemoteVersionEvent) {
            eventType = BPMEventType.VersionDemoted;
        } else if (event instanceof FileEvent) {
            eventType = BPMEventType.Filed;
        } else if (event instanceof FreezeEvent) {
            eventType = BPMEventType.Frozen;
        } else if (event instanceof LockEvent) {
            eventType = BPMEventType.Locked;
        } else if (event instanceof PromoteVersionEvent) {
            eventType = BPMEventType.VersionPromoted;
        } else if (event instanceof PublishCompleteEvent) {
            eventType = BPMEventType.PublishCompleted;
        } else if (event instanceof PublishRequestEvent) {
            eventType = BPMEventType.PublishRequested;
        } else if (event instanceof UnfileEvent) {
            eventType = BPMEventType.Unfiled;
        } else if (event instanceof UnlockEvent) {
            eventType = BPMEventType.Unlocked;
        } else if (event instanceof UpdateEvent) {
            eventType = BPMEventType.Updated;
        } else if (event instanceof UpdateSecurityEvent) {
            eventType = BPMEventType.SecurityUpdated;
        } else {
            if (logger.isLoggable(Level.WARNING)) logger.warning("The ObjectChangeEvent named " + event.get_Name() + " has been ignored as there is no corresponding BPM ECMEvent it can be mapped to.");
        }
        if (logger.isLoggable(Level.FINEST)) logger.exiting(CLASS, methodName, eventType);
        return eventType;
    }
    
}
